% g, cm, sec

addpath('../two_dim_fea');

% E = 0.1 MPa; c = 0.04 kPa s; rho = 1 g/cm^2
Young = 1.0*1e+6; c = 0.4*1e+3; nu = 0.48; density = 1.00;
[ lambda, mu ] = Lame_constants( Young, nu );

m = 5; n = 5;
[ points, triangles, rectangles ] = rectangular_object( m, n, 10, 10 );
thickness = 1;

npoints = size(points,2);
nrectangles = size(rectangles,1);
elastic = Body(npoints, points, [], [], thickness);
elastic = elastic.rectangle_elements(nrectangles, rectangles);
elastic = elastic.mechanical_parameters(density, lambda, mu);
elastic = elastic.calculate_stiffness_matrix;
K = elastic.Stiffness_Matrix;

A = elastic.constraint_matrix([1:5,21:25]);
zr = [0;0]; ex = [1;0]; ey = [0;1];
d = [zr; zr; zr; zr; zr; ey; ey; ey; ey; ey];

% 10.0 N --> fpush = 10.0*1e+5;
force = [];
disp = [];

filename = 'push_test_rectangle_elements';

figure('position', [0, 0, 800, 800]);
set(0,'defaultAxesFontSize',32);
set(0,'defaultTextFontSize',32);

clf;
elastic.draw_individual_rectangles;
xlim([-2,12]);
ylim([-2,12]);
xticks([-2:2:12]);
grid on;
pbaspect([1 1 1]);
saveas(gcf, strcat(filename, '_natural.png'), 'png');

for fpushscaled = 0.0:1.0:20.0
    fpush = fpushscaled * 1e+5;
    mat = [ K, zeros(2*npoints,1), -A;
            zeros(1,2*npoints), 0, -d';
            -A', -d, zeros(20,20) ];
    vec = [ zeros(2*npoints,1); fpush; zeros(20,1) ];
    sol = mat \ vec;

    force = [force; fpushscaled];
    disp = [disp; sol(2*npoints+1)];
end

clf;
elastic.draw_individual_rectangles(reshape(sol(1:2*npoints), [2,npoints]));
xlim([-2,12]);
ylim([-2,12]);
xticks([-2:2:12]);
grid on;
pbaspect([1 1 1]);
saveas(gcf, strcat(filename, '_deformed.png'), 'png');
