% g, cm, sec

addpath('../two_dim_fea');

% E = 0.1 MPa; c = 0.04 kPa s; rho = 1 g/cm^2
Young = 1.0*1e+6; c = 0.4*1e+3; nu = 0.48; density = 1.00;

[ lambda, mu ] = Lame_constants( Young, nu );

m = 7; n = 4;
[ points, triangles ] = rectangular_object( m, n, 20, 10 );
thickness = 1;

npoints = size(points,2);
ntriangles = size(triangles,1);
elastic = Body(npoints, points, ntriangles, triangles, thickness);
elastic = elastic.mechanical_parameters(density, lambda, mu);
elastic = elastic.calculate_stiffness_matrix;

% 10.0 N --> fext = 10.0*1e+5;
%fext = [ 20; 10 ] * 1e+5;
fext = [ 10; 5 ] * 1e+5;
f = zeros(2*npoints,1);
f(2*14-1:2*14) = fext/2;
f(2*21-1:2*21) = fext/2;

A = zeros(2*npoints, 2*4);
A(2*1-1:2*1, 1:2) = eye(2);
A(2*8-1:2*8, 3:4) = eye(2);
A(2*15-1:2*15, 5:6) = eye(2);
A(2*22-1:2*22, 7:8) = eye(2);

mat = [ elastic.Stiffness_Matrix, -A; -A', zeros(8,8) ];
vec = [ f; zeros(8,1) ];
sol = mat \ vec;
un = sol(1:2*npoints);

figure('position', [0, 0, 800, 800]);
set(0,'defaultAxesFontSize',32);
set(0,'defaultTextFontSize',32);

clf;
elastic.draw(zeros(2, npoints));
xlim([-5,25]); ylim([-5,25]); grid on; pbaspect([1 1 1]);
saveas(gcf, 'simple_example_7_4_natural.png');

clf;
elastic.draw(reshape(un, [2, npoints]));
xlim([-5,25]); ylim([-5,25]); grid on; pbaspect([1 1 1]);
saveas(gcf, 'simple_example_7_4_deformed.png');
